
/*-===========================================================================--
--- MODULE   : uscib0_i2c_drv.c
--- VERSION  : 2.20
---===========================================================================---
--- Environment : Device      : MSP430G2553
---   Compiler/Assembler/IDE : CCS V5.2
---=========================================================================---*/
#include "msp430g2553.h"
#include "application.h"
#include "spi_il.h"
#include "stdtypes.h"

//***** Definitions *****
typedef enum
{
	OK = 0,
	ERROR
}I2C_BusState_T;

//***** Function Prototypes *****
#ifndef USING_I2C_INT
I2C_BusState_T USCIB0RX_ISR_Poll(void);
I2C_BusState_T USCIB0TX_ISR_Poll(void);
#endif

//***** Global Data *****
u08 locSlaveAddr;
u08 TXByteCtr;
u08 RXByteCtr;
u08 *PTxData;
u08 *PRxData;
Boolean MultiByteReadFG = FALSE;
Boolean WaitforSTPCondition = FALSE;
#ifdef USING_I2C_INT
Boolean RPTCondition = FALSE;
#endif

/***********************************************************
  Function: USCIB0_I2C_Init
*/
void USCIB0_I2C_Init(void)
{
	P1SEL |= BIT6 + BIT7;                     // Assign I2C pins to USCI_B0
	P1SEL2|= BIT6 + BIT7;                     // Assign I2C pins to USCI_B0

	IE2 &= ~UCB0RXIE;  						  // Disable RX interrupt
	IE2 &= ~UCB0TXIE;  						  // Disable TX interrupt

	UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
	UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset

	UCB0BR0 = 0xA0;                           // fSCL = SMCLK/160 = ~100kHz
//	UCB0BR0 = 0x28;                           // fSCL = SMCLK/40 = ~400kHz
	UCB0BR1 = 0;
}

/***********************************************************
  Function: USCIB0_I2C_Ready
*/
I2C_BusState_T USCIB0_I2C_Ready(void)
{
  // Configure a 50ms Timeout
  Timeout(5);

  while ( ( UCB0STAT & UCBBUSY ) && (Timer_CCR1_Get_Timeout_Counts()) );

  if (Timer_CCR1_Get_Timeout_Counts() != 0)
  {
	return (OK);
  }
  else
  {
    Timer_CCR1_Stop_Timeout();
	return (ERROR);
  }
}

/***********************************************************
  Function: USCIB0_I2C_SetSlaveAddress
*/
u08 USCIB0_I2C_SetSlaveAddress(u08 address)
{
	u08 returnValue = 0;
#ifdef USING_I2C_INT
	u08 ie2_bak;
	u08 slaveadr_bak;
	u08 ucb0i2cie;

    /* Verify that Slave is present */
	ucb0i2cie = UCB0I2CIE;                      // store old UCB0I2CIE
	ie2_bak = IE2;                              // store IE2 register
	slaveadr_bak = UCB0I2CSA;                   // store old slave address

	UCB0CTL1 |= UCSWRST;                        // Enable SW reset
	UCB0I2CIE &= ~ UCNACKIE;                    // no NACK interrupt
	UCB0I2CSA = address;                        // set slave address
	UCB0CTL1 &= ~UCSWRST;                       // Clear SW reset, resume operation

	IE2 &= ~(UCB0TXIE + UCB0RXIE);              // no RX or TX interrupts

	// Configure a 50ms Timeout
	Timeout(5);

	UCB0TXBUF = 0xFF;
	UCB0CTL1 |= UCTR + UCTXSTT + UCTXSTP;       // I2C TX, start and stop condition
	while ( (UCB0CTL1 & UCTXSTP) && (Timer_CCR1_Get_Timeout_Counts()) );             // wait for STOP condition

	if (Timer_CCR1_Get_Timeout_Counts())
	{
		returnValue = !(UCB0STAT & UCNACKIFG);
		TXByteCtr = 0;
		RXByteCtr = 0;
	}
	else
	{
		Timer_CCR1_Stop_Timeout();
	}

	UCB0CTL1 |= UCSWRST;                        // Enable SW reset
	IE2 = ie2_bak;                              // restore IE2
	UCB0I2CSA = slaveadr_bak;                   // restore old slave address
	UCB0I2CIE = ucb0i2cie;                      // restore old UCB0CIE
#else
	UCB0CTL1 |= UCSWRST;                        // Enable SW reset
	UCB0I2CIE &= ~ UCNACKIE;                    // no NACK interrupt
	UCB0I2CSA = address;                        // set slave address
	UCB0CTL1 &= ~UCSWRST;                       // Clear SW reset, resume operation

	// Configure a 50ms Timeout
	Timeout(5);

	UCB0TXBUF = 0xFF;
	UCB0CTL1 |= UCTR + UCTXSTT + UCTXSTP;       // I2C TX, start and stop condition
	while ( (UCB0CTL1 & UCTXSTP) && (Timer_CCR1_Get_Timeout_Counts()) );             // wait for STOP condition

	if (Timer_CCR1_Get_Timeout_Counts())
	{
		returnValue = !(UCB0STAT & UCNACKIFG);
		TXByteCtr = 0;
		RXByteCtr = 0;
	}
	else
	{
		Timer_CCR1_Stop_Timeout();
	}

	UCB0CTL1 |= UCSWRST;                        // Enable SW reset
#endif
	/* Set the new Slave address*/
	locSlaveAddr = address;

	return (returnValue);                         // return whether or not a NACK occurred
}

#ifdef USING_I2C_INT
/***********************************************************
  Function: USCIB0_I2C_TX_Init
*/
void USCIB0_I2C_TX_Init(u08 SlaveAddress)
{
	IE2 &= ~UCB0RXIE;

	UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	UCB0I2CSA = SlaveAddress;                 // Slave Address
	UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation

	IE2 |= UCB0TXIE;                          // Enable TX interrupt
}

/***********************************************************
  Function: USCIB0_I2C_RX_Init
*/
void USCIB0_I2C_RX_Init(u08 SlaveAddress)
{
	IE2 &= ~UCB0TXIE;

	UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
	UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
	UCB0I2CSA = SlaveAddress;                 // Slave Address
	UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation

	IE2 |= UCB0RXIE;                          // Enable RX interrupt
}

/***********************************************************
  Function: USCIB0_I2C_TX_MultiByte
*/
u08 USCIB0_I2C_TX_MultiByte(u08 *i2cArray, u08 NumBytes)
{
	I2C_BusState_T locBusState = ERROR;
	Boolean locRPTCondition = FALSE;
	u08 ret = 1;

	// Store a copy of RPTCondition
	locRPTCondition = RPTCondition;

	// Configure a 50ms Timeout
	Timeout(5);

	// Wait for bus to be free
	while ( (UCB0CTL1 & UCTXSTP) || ( (UCB0STAT & UCBBUSY) && (Timer_CCR1_Get_Timeout_Counts()) ) );

	if (Timer_CCR1_Get_Timeout_Counts())
	{
		USCIB0_I2C_TX_Init(locSlaveAddr);         // re-init i2c with new slave address

		// Wait for bus to be free
		locBusState = USCIB0_I2C_Ready();
		if ( locBusState == OK )
		{
			/* start transmitting */
			PTxData = i2cArray;                       // TX array start address
			TXByteCtr = NumBytes;                     // Load TX byte counter

			UCB0CTL1 |= UCTR + UCTXSTT;               // I2C TX, start condition

			// Reset Timeout 50ms
			Timeout(5);

			while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

			if (Timer_CCR1_Get_Timeout_Counts() != 0)
			{
				// Reset Timeout 50ms
				Timeout(5);

				// Ensure stop condition got sent
				while ( (!WaitforSTPCondition) && (Timer_CCR1_Get_Timeout_Counts()) );

				if (!locRPTCondition)
				{
					// Avoid STOP condition overrun if multiple i2c are sent one after the other (contiguously)
					while ( (UCB0CTL1 & UCTXSTP) && (Timer_CCR1_Get_Timeout_Counts()) );        // wait for STOP condition
				}
			}
		}
	}
	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
    {
    	/* Send I2C to bus off */
    	Send_I2C_to_BusOFF();
		ret = 0;
    }
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

    WaitforSTPCondition = FALSE;

	return (ret);
}

/***********************************************************
  Function: USCIB0_I2C_RX_MultiByte
*/
u08 USCIB0_I2C_RX_MultiByte(u08 *i2cArray, u08 RxNumBytes, u08 TxNumBytes)
{
	I2C_BusState_T locBusState = ERROR;
	u08 ret = 1;

	// Configure a 50ms Timeout
	Timeout(5);

	// Wait for bus to be free
	while (MultiByteReadFG || (UCB0CTL1 & UCTXSTP) || ( (UCB0STAT & UCBBUSY) && (Timer_CCR1_Get_Timeout_Counts()) ));

	if (Timer_CCR1_Get_Timeout_Counts())
	{
		MultiByteReadFG = TRUE;					  // Multi-byte transaction starting

		// Reset Timeout 50ms
		Timeout(5);

		RPTCondition = TRUE;
		USCIB0_I2C_TX_MultiByte(i2cArray,TxNumBytes);      // Write the read register to slave device
		USCIB0_I2C_RX_Init(locSlaveAddr);	               // Initialize I2C read operation

		PRxData = (i2cArray + TxNumBytes);                 // RX array start address

		if (RxNumBytes == 1)
		{
			RXByteCtr = 0;
			MultiByteReadFG = FALSE;
			UCB0CTL1 |= UCTXSTT;                    // Send I2C start condition

			while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

			if (Timer_CCR1_Get_Timeout_Counts() != 0)
			{
				UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
			}
		}
		else
		{
			RXByteCtr = RxNumBytes - 2;               // Load RX byte counter
			UCB0CTL1 |= UCTXSTT;                      // Send I2C start condition
		}

		// Wait until I2C is ready
		locBusState = USCIB0_I2C_Ready();
	}

	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
	{
		/* Send I2C to bus off */
		ret = 0;
		Send_I2C_to_BusOFF();
	}
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

	return (ret);
}

/***********************************************************
  Function: USCIB0_I2C_RX_SingleByte
*/
u08 USCIB0_I2C_RX_SingleByte(u08 *i2cArray)
{
	I2C_BusState_T locBusState = ERROR;
	u08 ret = 1;

	// Wait for bus to be free
	locBusState = USCIB0_I2C_Ready();

	if ( locBusState == OK )
	{
		USCIB0_I2C_RX_Init(locSlaveAddr);	    // Initialize I2C read operation

		// Set Rx Buffer
		PRxData = i2cArray;

		// Clear Rx Counter
		RXByteCtr = 0;

		UCB0CTL1 |= UCTXSTT;                    // Send I2C start condition

		// Configure a 50ms Timeout
		Timeout(5);

		while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

		if (Timer_CCR1_Get_Timeout_Counts() != 0)
		{
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition

			// Wait until I2C is ready
			locBusState = USCIB0_I2C_Ready();
		}
	}

	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
	{
		/* Send I2C to bus off */
		ret = 0;
		Send_I2C_to_BusOFF();
	}
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

	return (ret);
}

/***********************************************************
  Function: USCIB0TX_ISR
*/
void USCIB0TX_ISR(void)
{
	if (IFG2 & UCB0RXIFG)				// RX - if a receive interrupt (byte received in buffer)
	{
	    if ( RXByteCtr > 0 )
	    {
			*PRxData = UCB0RXBUF;             // Load RX buffer
			PRxData++;
	        RXByteCtr--;                      // Decrement TX byte counter
	    }
	    else if ( (RXByteCtr == 0) && (MultiByteReadFG) )                    // Check RX byte counter
		{
			*PRxData = UCB0RXBUF;                   // Load RX buffer
			PRxData++;
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
			MultiByteReadFG = FALSE;				// Multi-byte transaction done
		}
		else
		{
			*PRxData = UCB0RXBUF;                   // Load RX buffer
		}
	}
	else
	{
		if (TXByteCtr)                        // Check TX byte counter
		{
			UCB0TXBUF = *PTxData;                   // Load TX buffer
			PTxData++;
			TXByteCtr--;                            // Decrement TX byte counter
		}
		else if (RPTCondition)
		{
			UCB0TXBUF = 0xFF;                       // Dummy write, it will not go out due to the RPT condition
			RPTCondition = FALSE;
		}
		else
		{
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
			WaitforSTPCondition = TRUE;
			IFG2 &= ~UCB0TXIFG;                     // Clear USCI_B0 TX int flag
		}
	}
}

#else
/***********************************************************
  Function: USCIB0_I2C_TX_Init_Poll
*/
void USCIB0_I2C_TX_Init_Poll(u08 SlaveAddress)
{
	IFG2 &= ~UCB0TXIFG;                       // Clear USCI_B0 TX int flag

	UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	UCB0I2CSA = SlaveAddress;                 // Slave Address
	UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
}

/***********************************************************
  Function: USCIB0_I2C_RX_Init_Poll
*/
void USCIB0_I2C_RX_Init_Poll(u08 SlaveAddress)
{
	IFG2 &= ~UCB0RXIFG;                       // Clear USCI_B0 RX int flag

	UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
	UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
	UCB0I2CSA = SlaveAddress;                 // Slave Address
	UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
}

/***********************************************************
  Function: USCIB0_I2C_TX_MultiByte_Poll
*/
u08 USCIB0_I2C_TX_MultiByte_Poll(u08 *i2cArray, u08 NumBytes)
{
	I2C_BusState_T locBusState = ERROR;
	u08 ret = 1;

	USCIB0_I2C_TX_Init_Poll(locSlaveAddr);         // re-init i2c with new slave address

	// Wait for bus to be free
	locBusState = USCIB0_I2C_Ready();

	if ( locBusState == OK )
	{
		/* start transmitting */
		PTxData = i2cArray;                       // TX array start address
		TXByteCtr = NumBytes;                     // Load TX byte counter

		UCB0CTL1 |= UCTR + UCTXSTT;               // I2C TX, start condition

		UCB0TXBUF = *PTxData;                     // Load TX buffer
		PTxData++;
		TXByteCtr--;                              // Decrement TX byte counter

		// Reset Timeout 50ms
		Timeout(5);

		while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

		if (TXByteCtr != 0)
		{			
            // Poll ISR
			locBusState = USCIB0TX_ISR_Poll();
		}
		else
		{
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
		}

		if (Timer_CCR1_Get_Timeout_Counts() != 0)
		{
			// Reset Timeout 50ms
			Timeout(5);

			// Ensure stop condition got sent
			while ( (UCB0CTL1 & UCTXSTP) && (Timer_CCR1_Get_Timeout_Counts()) );
		}
	}

	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
    {
    	/* Send I2C to bus off */
    	Send_I2C_to_BusOFF();
		ret = 0;
    }
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

	return (ret);
}

/***********************************************************
  Function: USCIB0_I2C_RX_MultiByte_Poll
*/
u08 USCIB0_I2C_RX_MultiByte_Poll(u08 *i2cArray, u08 RxNumBytes, u08 TxNumBytes)
{
	I2C_BusState_T locBusState = ERROR;
	u08 ret = 1;

	// Configure a 50ms Timeout
	Timeout(5);

	// Wait for bus to be free
	while (MultiByteReadFG || (UCB0CTL1 & UCTXSTP) || ( (UCB0STAT & UCBBUSY) && (Timer_CCR1_Get_Timeout_Counts()) ));

	if (Timer_CCR1_Get_Timeout_Counts())
	{
		MultiByteReadFG = TRUE;					  // Multi-byte transaction starting

		// Reset Timeout 50ms
		Timeout(5);

		USCIB0_I2C_TX_MultiByte_Poll(i2cArray,TxNumBytes);     // Write the read register to slave device
		USCIB0_I2C_RX_Init_Poll(locSlaveAddr);	               // Initialize I2C read operation

		PRxData = (i2cArray + TxNumBytes);                     // RX array start address

		if (RxNumBytes == 1)
		{
			RXByteCtr = 0;
			MultiByteReadFG = FALSE;
			UCB0CTL1 |= UCTXSTT;                    // Send I2C start condition

			while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition

			// Poll ISR
			locBusState = USCIB0RX_ISR_Poll();

			if (Timer_CCR1_Get_Timeout_Counts() != 0)
			{
				// Wait until I2C is ready
				locBusState = USCIB0_I2C_Ready();
				IFG2 &= ~UCB0RXIFG;                     // Clear USCI_B0 RX int flag
			}
		}
		else
		{
			RXByteCtr = RxNumBytes - 2;               // Load RX byte counter
			UCB0CTL1 |= UCTXSTT;                      // Send I2C start condition

			while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

			// Poll ISR
			locBusState = USCIB0RX_ISR_Poll();
		}

		// Wait until I2C is ready
		locBusState = USCIB0_I2C_Ready();
	}

	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
	{
		/* Send I2C to bus off */
		ret = 0;
		Send_I2C_to_BusOFF();
	}
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

	return (ret);
}

/***********************************************************
  Function: USCIB0_I2C_RX_SingleByte_Poll
*/
u08 USCIB0_I2C_RX_SingleByte_Poll(u08 *i2cArray)
{
	I2C_BusState_T locBusState = ERROR;
	u08 ret = 1;

	// Wait for bus to be free
	locBusState = USCIB0_I2C_Ready();

	if ( locBusState == OK )
	{
		USCIB0_I2C_RX_Init_Poll(locSlaveAddr);	    // Initialize I2C read operation

		// Set Rx Buffer
		PRxData = i2cArray;

		// Clear Rx Counter
		RXByteCtr = 0;

		UCB0CTL1 |= UCTXSTT;                    // Send I2C start condition

		// Configure a 50ms Timeout
		Timeout(5);
		while ( (UCB0CTL1 & UCTXSTT) && (Timer_CCR1_Get_Timeout_Counts()) );

		// Poll ISR
		UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
		locBusState = USCIB0RX_ISR_Poll();

		if (Timer_CCR1_Get_Timeout_Counts() != 0)
		{
			// Wait until I2C is ready
			locBusState = USCIB0_I2C_Ready();
			IFG2 &= ~UCB0RXIFG;                     // Clear USCI_B0 RX int flag
		}
	}

	// Check if there was a TimeOut
    if ( (!Timer_CCR1_Get_Timeout_Counts()) || (locBusState == ERROR) )
	{
		/* Send I2C to bus off */
		ret = 0;
		Send_I2C_to_BusOFF();
	}
    else
    {
    	Timer_CCR1_Stop_Timeout();
    }

	return (ret);
}

/***********************************************************
  Function: USCIB0RX_ISR_Poll
*/
I2C_BusState_T USCIB0RX_ISR_Poll(void)
{
	Boolean Rx_ISR = TRUE;

   	// Reset Timeout 50ms
    Timeout(5);

	while (Rx_ISR)
	{
        // RX - if a receive interrupt (byte received in buffer)
		while ( ((IFG2 & UCB0RXIFG) == 0x00 ) && (Timer_CCR1_Get_Timeout_Counts()) );

        if (!Timer_CCR1_Get_Timeout_Counts())
        {
          return (ERROR);
        }

		IFG2 &= ~UCB0RXIFG;                     // Clear USCI_B0 RX int flag

		if ( RXByteCtr > 0 )
		{
			*PRxData = UCB0RXBUF;               // Load RX buffer
			PRxData++;
			RXByteCtr--;                        // Decrement TX byte counter
		}
		else if ( (RXByteCtr == 0) && (MultiByteReadFG) )                    // Check RX byte counter
		{
			*PRxData = UCB0RXBUF;                   // Load RX buffer
			PRxData++;
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
			MultiByteReadFG = FALSE;				// Multi-byte transaction done
			// Rx_ISR  = FALSE;
		}
		else
		{
			*PRxData = UCB0RXBUF;                   // Load RX buffer
			Rx_ISR  = FALSE;
		}
	}

	return (OK);
}


/***********************************************************
  Function: USCIB0TX_ISR_Poll
*/
I2C_BusState_T USCIB0TX_ISR_Poll(void)
{
	Boolean Tx_ISR = TRUE;

   	// Reset Timeout 50ms
    Timeout(5);

	while (Tx_ISR)
	{
        // TX - Wait for the data is transferred from the buffer into the shift register
		while ( ((IFG2 & UCB0TXIFG) == 0x00 ) && (Timer_CCR1_Get_Timeout_Counts()) );
        
        if (!Timer_CCR1_Get_Timeout_Counts())
        {
          return (ERROR);
        }
        
		IFG2 &= ~UCB0TXIFG;                     // Clear USCI_B0 TX int flag

		if (TXByteCtr > 0)                      // Check TX byte counter
		{
			UCB0TXBUF = *PTxData;               // Load TX buffer
			PTxData++;
			TXByteCtr--;                        // Decrement TX byte counter
		}
		else
		{
			UCB0CTL1 |= UCTXSTP;                // I2C stop condition
			Tx_ISR = FALSE;
		}
	}
    
    return (OK);
}
#endif
/*-===========================================================================--
--- Version Information
--- 1.00 (08/31/2012) - Initial Release
--- 1.10 (10/26/2012) - Check for the I2C slave before sending any message in
---                     USCIB0_I2C_SetSlaveAddress, change it from void to u08
--- 1.20 (11/14/2012) - Added an extra parameter on USCIB0_I2C_RX_MultiByte
---                     (TxNumBytes), if more than 1 byte is needed before
---                     the repeated start
---                     Created USCIB0_I2C_RXByte_Test, for FuelGauge Debug
--- 1.30 (11/20/2012) - Removed USCIB0_I2C_RXByte_Test and created
---                     USCIB0_I2C_RX_SingleByte, used by the FuelGauge
--- 1.40 (12/07/2012) - Reduced I2C clock to 100KHz, for testing purposes
--- 1.50 (12/14/2012) - Replaced the delays on USCIB0_I2C_RX_MultiByte and
---                     USCIB0_I2C_RX_SingleByte with USCIB0_I2C_Ready.
--- 1.60 (12/21/2012) - Updated the I2C BusOff implementation.
--- 1.70 (01/30/2013) - Added Boot Loader support. Relocated interrupt vectors
---                     to app_vect.c
--- 1.80 (02/25/2013) - Changed Shared_Data to AppFlags.
--- 1.90 (03/26/2013) - Speed-up that Slave is present check
--- 2.00 (04/10/2013) - Changed variable name AppFlags to StatusFlags.
--- 2.10 (04/22/2013) - Implemented I2C timeouts, avoid "hanging" condition.
--- 2.20 (06/21/2013) - Included PACE updates from SW V4.21.
---
---=========================================================================---*/
